@@ -14,7 +14,6 @@
 
 using namespace Qt::StringLiterals;
 
-const QLatin1StringView protobuftyperegistrations("_protobuftyperegistrations.cpp");
 const QLatin1StringView cppExtension(".qpb.cpp");
 const QLatin1StringView headerExtension(".qpb.h");
 const QLatin1StringView protocGenQtprotobufKey(" --plugin=protoc-gen-qtprotobuf=");
@@ -175,7 +174,8 @@ void tst_qtprotobufgen::initTestCase()
     QDir testOutputBaseDir(QCoreApplication::applicationDirPath());
     testOutputBaseDir.mkdir(QLatin1StringView("cmd_line_generation"));
     QLatin1StringView folders[] = {"comments"_L1, "extra-namespace"_L1,
-                                   "fieldenum"_L1, "folder"_L1, "no-options"_L1};
+                                   "fieldenum"_L1, "folder"_L1,
+                                    "qml-no-package"_L1, "no-options"_L1};
     for (QLatin1StringView folder : folders)
         testOutputBaseDir.mkdir("cmd_line_generation/"_L1 + folder);
 
@@ -196,8 +196,7 @@ void tst_qtprotobufgen::cmakeGeneratedFile_data()
     QTest::addColumn<QString>("folder");
     QTest::addColumn<QString>("extension");
 
-    const QLatin1StringView extensions[]
-            = {protobuftyperegistrations, cppExtension, headerExtension};
+    const QLatin1StringView extensions[] = {cppExtension, headerExtension};
 
     for (const auto extension : extensions) {
         QTest::addRow("repeatednonpackedmessages%s", extension.data())
@@ -220,6 +219,16 @@ void tst_qtprotobufgen::cmakeGeneratedFile_data()
                 << "/folder/qtprotobufnamespace/tests/"
                 << QString(extension);
 
+        QTest::addRow("oneofmessages%s", extension.data())
+                << "oneofmessages"
+                << "/folder/qtprotobufnamespace/tests/"
+                << QString(extension);
+
+        QTest::addRow("optional%s", extension.data())
+                << "optional"
+                << "/folder/qtprotobufnamespace/optional/tests/"
+                << QString(extension);
+
         QTest::addRow("repeatedmessages%s", extension.data())
                 << "repeatedmessages"
                 << "/folder/qtprotobufnamespace/tests/"
@@ -234,7 +243,30 @@ void tst_qtprotobufgen::cmakeGeneratedFile_data()
                 << "extranamespace"
                 << "/extra-namespace/"
                 << QString(extension);
+#ifdef HAVE_QML
+        QTest::addRow("nopackage%s", extension.data())
+                << "nopackage"
+                << "/qml-no-package/"
+                << QString(extension);
+#endif
     }
+#ifdef HAVE_QML
+    const QLatin1StringView qmlExtensions[]
+            = {cppExtension,
+                headerExtension};
+
+    for (const auto extension : qmlExtensions) {
+        QTest::addRow("enummessages%s with QML option", extension.data())
+                << "enummessages"
+                << "/qmlgen/"
+                << QString(extension);
+
+        QTest::addRow("basicmessages%s with QML option", extension.data())
+                << "basicmessages"
+                << "/qmlgen/"
+                << QString(extension);
+    }
+#endif
 }
 
 void tst_qtprotobufgen::cmakeGeneratedFile()
@@ -280,7 +312,7 @@ void tst_qtprotobufgen::cmdLineGeneratedFile_data()
     QTest::addColumn<QString>("generatedFolderStructure");
 
     const QLatin1StringView extensions[]
-            = {protobuftyperegistrations, cppExtension, headerExtension};
+            = {cppExtension, headerExtension};
 
     for (const auto extension : extensions) {
         QTest::addRow("basicmessages%s", extension.data())
@@ -297,6 +329,20 @@ void tst_qtprotobufgen::cmdLineGeneratedFile_data()
                 << QString(extension)
                 << "qtprotobufnamespace/tests/";
 
+        QTest::addRow("oneofmessages%s", extension.data())
+                << "oneofmessages"
+                << "GENERATE_PACKAGE_SUBFOLDERS"
+                << "/folder/"
+                << QString(extension)
+                << "qtprotobufnamespace/tests/";
+
+        QTest::addRow("optional%s", extension.data())
+                << "optional"
+                << "GENERATE_PACKAGE_SUBFOLDERS"
+                << "/folder/"
+                << QString(extension)
+                << "qtprotobufnamespace/optional/tests/";
+
         QTest::addRow("repeatedmessages%s", extension.data())
                 << "repeatedmessages"
                 << "GENERATE_PACKAGE_SUBFOLDERS"
@@ -324,6 +370,14 @@ void tst_qtprotobufgen::cmdLineGeneratedFile_data()
                 << "/extra-namespace/"
                 << QString(extension)
                 << "";
+#ifdef HAVE_QML
+        QTest::addRow("nopackage%s", extension.data())
+                << "nopackage"
+                << "QML_URI=nopackage.uri.test;EXPORT_MACRO=TST_QTPROTOBUFGEN_NOPACKAGE_QML_GEN"
+                << "/qml-no-package/"
+                << QString(extension)
+                << "";
+#endif
     }
 }
 
@@ -393,7 +447,7 @@ void tst_qtprotobufgen::cmdLineGeneratedNoOptions_data()
     QTest::addColumn<QString>("extension");
 
     const QLatin1StringView extensions[]
-            = {protobuftyperegistrations, cppExtension, headerExtension};
+            = {cppExtension, headerExtension};
 
     for (const auto extension : extensions) {
         QTest::addRow("annotation%s", extension.data())
@@ -421,6 +475,16 @@ void tst_qtprotobufgen::cmdLineGeneratedNoOptions_data()
                 << "/no-options/"
                 << QString(extension);
 
+        QTest::addRow("oneofmessages%s", extension.data())
+                << "oneofmessages"
+                << "/no-options/"
+                << QString(extension);
+
+        QTest::addRow("optional%s", extension.data())
+                << "optional"
+                << "/no-options/"
+                << QString(extension);
+
         QTest::addRow("repeatedmessages%s", extension.data())
                 << "repeatedmessages"
                 << "/no-options/"
@@ -430,6 +494,11 @@ void tst_qtprotobufgen::cmdLineGeneratedNoOptions_data()
                 << "repeatednonpackedmessages"
                 << "/no-options/"
                 << QString(extension);
+
+        QTest::addRow("anymessages%s", extension.data())
+                << "anymessages"
+                << "/no-options/"
+                << QString(extension);
     }
 }
 
